/*
 * Simple VGA (640x480) analog clock
 *
 * Dave Dunfield - in response to posting on Vogons - Feb 2, 2003
 *	https://dunfield.themindfactory.com
 */
#include <stdio.h>
#include <hrg.h>	// HighResGraphcs

#define	ARC_RES	64	// Arc resolution

#define	CX	320		// Center X
#define	CY	240		// Center Y
#define	CR	220		// Circle Radius
#define	HR	100		// Hour Radius
#define	MR	150		// Minite Radios
#define	SR	190		// Second Radius

#define	CO	7		// Color

unsigned
	H, M, S,
	X, Y;

unsigned Sine[] = {		// Sine table
	    0,  1608,  3216,  4821,  6424,  8022,  9616, 11204,
	12785, 14359, 15924, 17479, 19024, 20557, 22078, 23586,
	25079, 26557, 28020, 29465, 30893, 32302, 33692, 35062,
	36410, 37736, 39040, 40320, 41575, 42806, 44011, 45190,
	46341, 47464, 48559, 49624, 50660, 51665, 52639, 53581,
	54491, 55368, 56212, 57022, 57797, 58538, 59243, 59913,
	60547, 61144, 61705, 62228, 62714, 63162, 63571, 63943,
	64276, 64571, 64826, 65043, 65220, 65358, 65457, 65516 };

// Calculate X,Y point from center given a 0-255 angle and radius
void Xpoint(int a, int r)
{
	int rs, i, j, ax;

	rs = r*r;

	j = (ARC_RES-1) - (i = a & (ARC_RES-1));
	switch(a & (ARC_RES*3)) {
	case ARC_RES*0 :	/* Quadrant one */
		X = CX + (ax = hrg_scale(r, Sine[i], -1));
		Y = CY - sqrt(rs - (ax*ax));
		break;
	case ARC_RES*1 :	/* Quadrant two */
		X = CX + (ax = hrg_scale(r, Sine[j], -1));
		Y = CY + sqrt(rs - (ax*ax));
		break;
	case ARC_RES*2 :	/* Quadrant three */
		X = CX - (ax = hrg_scale(r, Sine[i], -1));
		Y = CY + sqrt(rs - (ax*ax));
		break;
	case ARC_RES*3 :	/* Quadrant four */
		X = CX - (ax = hrg_scale(r, Sine[j], -1));
		Y = CY - sqrt(rs - (ax*ax)); }
}

// Update H/M/S lines
void UpTime(unsigned color)
{
	Xpoint(H, HR);
	hrg_line(CX, CY, X, Y, color);
	Xpoint(M, MR);
	hrg_line(CX, CY, X, Y, color);
	Xpoint(S, SR);
	hrg_line(CX, CY, X, Y, color);
}

main()
{
	int i, j, k, l;

	hrg_open();

	hrg_circle(CX, CY, CR, CO);		// Draw clock circle

	hrg_puts( 10, 470, CO, "ESC TO");
	hrg_puts(600, 470, CO, "EXIT");

	// Fill in hour numbers
	for(i=0; i < 12; ++i) {
		Xpoint(i * 21, CR-10);
		hrg_printf(X-5, Y-3, CO, "%u", i || 12); }

	do {
a1:		get_time(&i, &j, &k);
		if(k == l) {		// Second hasn't changed
			delay(100);		// Wait a bit
			goto a1; }		// and retry
		l = k;
		UpTime(0);
		if(i > 11) i -= 12;
		H = (i*2133)/100;	// Adjust for 256/12
//H = i * 21;
		M = (j*435)/100;	// "" 60/12
		S = (k*435)/100;	// ""
		UpTime(7); }
	while(kbtst() != 0x1B);

	hrg_close();
	
}
